import base64
import os
from configparser import ConfigParser
from enum import Enum
from typing import List, Optional, TypedDict, cast

import typer
from databricks_cli.sdk.api_client import ApiClient
from databricks_cli.sdk.service import WorkspaceService


class ObjectType(str, Enum):
    directory = "DIRECTORY"
    notebook = "NOTEBOOK"
    library = "LIBRARY"
    repo = "REPO"


class ListEntry(TypedDict):
    path: str
    object_type: ObjectType


class DatabricksAPI:
    def __init__(
        self,
        databricks_host: str,
        databricks_token: str,
        username: Optional[str] = None,
        verify_ssl: bool = True,
    ) -> None:
        self.client = WorkspaceService(
            ApiClient(host=databricks_host, token=databricks_token, verify=verify_ssl)
        )
        self.username = username

    def _resolve_path(self, path: str) -> str:
        """
        Resolve path to notebook.

        If `path` is not absolute and a username is in use, interpret the path as
        relative to the user direcotry.

        Example:

            tmp/notebook.py  -> /Users/{username}/tmp/notebook.py
            /tmp/notebook.py -> /tmp/notebook.py
        """
        if not path.startswith("/") and self.username is not None:
            path = f"/Users/{self.username}/{path}"
        return path

    def read_notebook(self, path: str) -> str:
        path = self._resolve_path(path)
        response = self.client.export_workspace(path, format="SOURCE")
        return base64.decodebytes(response["content"].encode()).decode()

    def write_notebook(self, path: str, content: str) -> None:
        path = self._resolve_path(path)
        self.client.import_workspace(
            path,
            format="SOURCE",
            language="PYTHON",
            content=base64.b64encode(content.encode()).decode(),
            overwrite=True,
        )

    def list_workspace(self, path: str) -> List[ListEntry]:
        return cast(List[ListEntry], self.client.list(path)["objects"])


def get_api_client(profile_name: str) -> DatabricksAPI:
    config = ConfigParser()
    config_path = os.path.expanduser("~/.databrickscfg")
    try:
        config.read(config_path)
    except FileNotFoundError:
        typer.echo(
            typer.style("Error:", fg=typer.colors.RED, bold=True)
            + typer.style(
                " Your ~/.databrickscfg file is missing.",
                bold=True,
            ),
        )
        typer.echo(
            "If you haven't already, first install the databricks cli,\n"
            "then run `databricks configure` and check your ~/.databrickscfg file."
        )
        raise typer.Abort()
    except Exception as e:
        typer.echo(
            typer.style("Error:", fg=typer.colors.RED, bold=True)
            + typer.style(
                " An error occurred while reading your ~/.databrickscfg file.",
                bold=True,
            )
        )
        raise e

    if profile_name != "DEFAULT" and not config.has_section(profile_name):
        typer.echo(
            typer.style("Error:", fg=typer.colors.RED, bold=True)
            + typer.style(
                f" Could not find {profile_name} in ~/.databrickscfg", bold=True
            ),
        )
        typer.echo(
            "Run `databricks configure` and/or check your ~/.databrickscfg file."
        )
        raise typer.Abort()

    host = config.get(profile_name, "host")  # Will throw if missing, always needed.
    username = config.get(profile_name, "username", fallback=None)
    token = config.get(profile_name, "token", fallback=None)
    password = config.get(profile_name, "password", fallback=None)
    insecure = config.get(profile_name, "insecure", fallback=None)
    credentials = token if token is not None else password

    # Handle no username:
    if username is None:
        typer.echo(
            typer.style("Warning:", fg=typer.colors.YELLOW, bold=True)
            + typer.style(" No username in ~/.databrickscfg.", bold=True)
        )
        typer.echo(
            "Please add `username = ...` to your profile (and/or the [DEFAULT] profile)"
            ", or be sure to always write full paths to notebooks."
        )

    # Handle no token and no password:
    if credentials is None:
        typer.echo(
            typer.style("Error:", fg=typer.colors.RED, bold=True)
            + typer.style(
                f" Found neither token nor password in ~/.databrickscfg for profile {profile_name}",
                bold=True,
            ),
        )
        raise typer.Abort()

    # Handle password without a username:
    if token is None and password is not None and username is None:
        typer.echo(
            typer.style("Error:", fg=typer.colors.RED, bold=True)
            + typer.style(
                f" Profile {profile_name} is missing a username in ~/.databrickscfg",
                bold=True,
            ),
        )
        raise typer.Abort()

    # Handle both token and password (one might be from DEFAULT):
    if token is not None and password is not None:
        if password != config.defaults().get("password"):
            credentials = password
        if token != config.defaults().get("token"):
            credentials = token

    return DatabricksAPI(
        databricks_host=host,
        databricks_token=credentials,
        username=username,
        verify_ssl=insecure is None,
    )



import os
import textwrap
import warnings
from typing import List, NoReturn, Optional, Sequence

import black
import typer

from . import __version__
from .blackbricks import HEADER, FormatConfig, format_str, unified_diff
from .databricks_sync import get_api_client
from .files import (
    File,
    LocalFile,
    RemoteNotebook,
    resolve_databricks_paths,
    resolve_filepaths,
)

app = typer.Typer(add_completion=False)


def process_files(
    files: Sequence[File],
    format_config: FormatConfig = FormatConfig(),
    diff: bool = False,
    check: bool = False,
) -> int:
    no_change = True
    n_changed_files = 0
    n_notebooks = 0

    for file_ in files:

        try:
            content = file_.content
        except UnicodeDecodeError:
            # File is not a text file. Probably a binary file. Skip.
            continue

        if not content.lstrip() or HEADER not in content.lstrip().splitlines()[0]:
            # Not a Databricks notebook - skip
            continue

        n_notebooks += 1
        output = format_str(content, config=format_config)

        no_change &= output == content
        n_changed_files += output != content

        if diff:
            diff_output = unified_diff(
                content,
                output,
                f"{os.path.basename(file_.path)} (before)",
                f"{os.path.basename(file_.path)} (after)",
            )
            if diff_output.strip():
                typer.echo(diff_output)
        elif not check:
            file_.content = output

            if output != content:
                typer.secho(f"reformatted {file_.path}", bold=True)
        elif check and output != content:
            typer.secho(f"would reformat {file_.path}", bold=True)

    unchanged_number = typer.style(
        str(n_notebooks - n_changed_files), fg=typer.colors.GREEN
    )
    changed_number = typer.style(str(n_changed_files), fg=typer.colors.MAGENTA)
    unchanged_echo = (
        f"{unchanged_number} files {'would be ' if check or diff else ''}left unchanged"
    )
    changed_echo = typer.style(
        f"{changed_number} files {'would be ' if check or diff else ''}reformatted",
        bold=True,
    )

    typer.secho("All done!", bold=True)
    typer.echo(
        ", ".join(
            filter(
                lambda s: bool(s),
                [
                    changed_echo if n_changed_files else "",
                    unchanged_echo if n_notebooks - n_changed_files > 0 else "",
                ],
            )
        )
    )
    return n_changed_files


def mutually_exclusive(names: List[str], values: List[bool]) -> None:
    if sum(values) > 1:
        names_formatted = ", ".join(
            typer.style(name, fg=typer.colors.CYAN) for name in names
        )
        typer.echo(
            f"{typer.style('Error:', fg=typer.colors.RED)} "
            + f"Only one of {names_formatted} may be use at the same time."
        )
        raise typer.Exit(1)


def version_callback(version_requested: bool) -> None:
    "Display versioin information and exit"
    if version_requested:
        version = typer.style(__version__, fg=typer.colors.GREEN)
        typer.echo(f"blackbricks, version {version}")
        raise typer.Exit()


@app.command()
def main(
    filenames: List[str] = typer.Argument(
        None, help="Path to the notebook(s) to format."
    ),
    remote_filenames: bool = typer.Option(
        False,
        "--remote",
        "-r",
        help="If this option is used, all filenames are treated as paths to "
        "notebooks on your Databricks host (i.e. not local files).",
    ),
    databricks_profile: str = typer.Option(
        "DEFAULT",
        "--profile",
        "-p",
        metavar="NAME",
        help="If using --remote, which Databricks profile to use.",
    ),
    line_length: int = typer.Option(
        black.const.DEFAULT_LINE_LENGTH, help="How many characters per line to allow."
    ),
    sql_upper: bool = typer.Option(
        True, help="SQL keywords should be UPPERCASE or lowercase."
    ),
    no_indent_with_two_spaces: Optional[bool] = typer.Option(
        None,
        "--no-indent-with-two-spaces",
        help="DEPRECATED: Blackbricks now uses 4 spaces for indentation by default. This option will be removed in future versions.",
    ),
    check: bool = typer.Option(
        False,
        "--check",
        help="Don't write the files back, just return the status. "
        "Return code 0 means nothing would change.",
        show_default=False,
    ),
    diff: bool = typer.Option(
        False,
        "--diff",
        help="Don't write the files back, just output a diff for each file on stdout.",
        show_default=False,
    ),
    version: bool = typer.Option(
        None,
        "--version",
        is_eager=True,
        callback=version_callback,
        help="Display version information and exit.",
    ),
) -> NoReturn:
    """
    Formatting tool for Databricks python notebooks.

    Python cells are formatted using `black`, and SQL cells are formatted by `sqlparse`.

    Local files (without the `--remote` option):

      - Only files that look like Databricks (Python) notebooks will be processed. That is, they must start with the header `# Databricks notebook source`

      - If you specify a directory as one of the file names, all files in that directory will be added, including any subdirectory.

    Remote files (with the `--remote` option):

      - Make sure you have installed the Databricks CLI (``pip install databricks_cli``)

      - Make sure you have configured at least one profile (`databricks configure`). Check the file `~/.databrickscfg` if you are not sure.

      - File paths should start with `/`. Otherwise they are interpreted as relative to `/Users/username`, where `username` is the username specified in the Databricks profile used.
    """
    assert not version, "If version is set, we don't get here."

    if no_indent_with_two_spaces is not None:
        warnings.simplefilter("always", DeprecationWarning)
        warnings.warn(
            textwrap.dedent(
                """
                Blackbricks now uses 4 spaces for indentation by default.
                Please stop using the `--no-indent-with-two-spaces` option, as it will be removed in future versions.
                """
            ),
            category=DeprecationWarning,
            stacklevel=0,
        )

    mutually_exclusive(["--check", "--diff"], [check, diff])

    if not filenames:
        typer.secho("No Path provided. Nothing to do.", bold=True)
        raise typer.Exit()

    files: List[File]
    if remote_filenames:
        api_client = get_api_client(databricks_profile)
        files = [
            RemoteNotebook(fname, api_client)
            for fname in resolve_databricks_paths(filenames, api_client=api_client)
        ]
    else:
        files = [LocalFile(fname) for fname in resolve_filepaths(filenames)]

    n_changed_files = process_files(
        files,
        format_config=FormatConfig(line_length=line_length, sql_upper=sql_upper),
        diff=diff,
        check=check,
    )
    raise typer.Exit(n_changed_files if check else 0)


if __name__ == "__main__":
    app()



from dataclasses import dataclass
from typing import Literal, Union

import black
import sqlparse

COMMAND = "# COMMAND ----------"
HEADER = "# Databricks notebook source"
NOFMT = "-- nofmt"
CELL_TITLE = "# DBTITLE 1,"


@dataclass(frozen=True)
class FormatConfig:
    "Data-only class to hold format configuration options and their defaults."
    line_length: int = black.const.DEFAULT_LINE_LENGTH
    sql_upper: bool = True


def format_str(content: str, config: FormatConfig = FormatConfig()) -> str:
    """
    Format the content of a notebook according to the format config provided.

    This assumes that `content` is the _full_ content of a notebook file, and that
    the notebook is a Python notebook.

    :param content: A string holding the entire content of a notebook.
    :param config: An object holding the desired formatting options.
    :return: The content of the file, formatted according to the configuration.
    """
    cells = content.replace(HEADER, "", 1).split(COMMAND)

    output_cells = []
    for cell in cells:
        cell = cell.strip()

        if "# MAGIC %sql" in cell:
            output_cells.append(
                _format_sql_cell(
                    cell, sql_keyword_case="upper" if config.sql_upper else "lower"
                )
            )
        elif "# MAGIC" in cell:
            output_cells.append(cell)  # Generic magic cell - output as-is.
        else:
            output_cells.append(
                black.format_str(
                    cell, mode=black.FileMode(line_length=config.line_length)
                )
            )

    output = (
        f"{HEADER}\n"
        + f"\n\n{COMMAND}\n\n".join(
            "".join(line.rstrip() + "\n" for line in cell.splitlines()).rstrip()
            for cell in output_cells
        ).rstrip()
    )

    # Databricks adds a space after '# MAGIC', regardless of wheter this constitutes
    # trailing whitespace. To avoid perpetual differences, blackbricks also adds this
    # extra whitespace. This is only added if the line does not contain anything else.
    # In all other cases, blackbricks will remove trailing whitespace.
    output_ws_normalized = ""
    for line in output.splitlines():
        line = line.rstrip()

        if line == "# MAGIC":
            line += " "

        output_ws_normalized += line + "\n"

    return output_ws_normalized.strip()


def _format_sql_cell(
    cell: str, sql_keyword_case: Union[Literal["upper"], Literal["lower"]] = "upper"
) -> str:
    """
    Format a MAGIC %sql cell.

    :param cell: The content of an SQL cell.
    :param sql_keyword_case: One of ["upper", "lower"], setting the case for SQL keywords.
    :return: The cell with formatting applied.
    """

    # Cells can have a title. This should just be kept at the start of the cell.
    if CELL_TITLE in cell.lstrip().splitlines()[0]:
        title_line = cell.lstrip().splitlines()[0] + "\n"
        cell = cell[len(title_line) :]
    else:
        title_line = ""

    # Formatting can be disabled on a cell-by-cell basis by adding `-- nofmt` to the first line.
    if NOFMT in cell.strip().splitlines()[0]:
        return title_line + cell

    magics = []
    sql_lines = []
    for line in cell.strip().splitlines():
        if line.strip().startswith("# MAGIC %sql"):
            continue
        words = line.split()
        magic, sql = words[:2], words[2:]
        magics.append(magic)
        sql_lines.append(" ".join(sql).strip())

    return (
        title_line
        + "# MAGIC %sql\n"
        + "\n".join(
            f"# MAGIC {sql}"
            for sql in sqlparse.format(
                "\n".join(sql_lines), reindent=True, keyword_case=sql_keyword_case
            ).splitlines()
        )
    )


def unified_diff(a: str, b: str, a_name: str, b_name: str) -> str:
    """
    Return a unified diff string between strings `a` and `b`.

    :param a: The first string (e.g. before).
    :param b: The second string (e.g. after).
    :param a_name: The "filename" to display for the `a` string.
    :param b_name: The "filename" to display for the `b` string.
    :return: A `git diff` like diff of the two strings.
    """
    import difflib

    a_lines = [line + "\n" for line in a.split("\n")]
    b_lines = [line + "\n" for line in b.split("\n")]
    return "".join(
        difflib.unified_diff(a_lines, b_lines, fromfile=a_name, tofile=b_name, n=5)
    )



import os
from dataclasses import dataclass
from typing import List

import typer

from .databricks_sync import DatabricksAPI, ObjectType


@dataclass
class File:
    path: str

    @property
    def content(self) -> str:
        raise NotImplementedError(
            "Do not use this class directly, use one of its subclasses."
        )

    @content.setter
    def content(self, _: str, /) -> None:
        raise NotImplementedError(
            "Do not use this class directly, use one of its subclasses."
        )


class LocalFile(File):
    @property
    def content(self) -> str:
        with open(self.path) as f:
            return f.read()

    @content.setter
    def content(self, new_content: str, /) -> None:
        with open(self.path, "w") as f:
            f.write(new_content)


@dataclass
class RemoteNotebook(File):
    api_client: DatabricksAPI

    @property
    def content(self) -> str:
        return self.api_client.read_notebook(self.path)

    @content.setter
    def content(self, new_content: str, /) -> None:
        self.api_client.write_notebook(self.path, new_content)


def resolve_filepaths(paths: List[str]) -> List[str]:
    """Resolve the paths given into valid file names

    Directories are recursively added, similarly to how black operates.

    :param paths: List of paths to files or directories.
    :return: Absolute paths to all files given, including all files in any
             directories given, including subdirectories.
    """
    file_paths = []
    while paths:
        path = os.path.abspath(paths.pop())

        if not os.path.exists(path):
            typer.echo(
                typer.style("Error:", fg=typer.colors.RED)
                + " No such file or directory: "
                + typer.style(path, fg=typer.colors.CYAN)
            )
            raise typer.Exit(1)

        if os.path.isdir(path):

            # Recursively  add all the files/dirs in path to the paths to be consumed.
            paths.extend([os.path.join(path, f) for f in os.listdir(path)])

        else:

            file_paths.append(path)

    return file_paths


def resolve_databricks_paths(
    paths: List[str], *, api_client: DatabricksAPI
) -> List[str]:
    """Resolve the remote paths given into valid file names

    Directories are recursively added, similarly to how black operates.

    :param paths: List of paths to remote files or directories.
    :api_client: Databricks API client to use.
    :return: Absolute paths to all files given, including all files in any
             directories given, including subdirectories.
    """
    paths = [api_client._resolve_path(path) for path in paths]
    file_paths = []
    while paths:
        path = paths.pop()
        response = api_client.list_workspace(path)
        for file_obj in response:
            if (obj_type := file_obj["object_type"]) == ObjectType.notebook.value:
                file_paths.append(file_obj["path"])
            elif obj_type in (ObjectType.directory.value, ObjectType.repo.value):
                paths.append(file_obj["path"])
    return file_paths



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['blackbricks']

package_data = \
{'': ['*']}

install_requires = \
['black>=22.8.0,<23.0.0',
 'databricks-cli>=0.17.3,<0.18.0',
 'sqlparse>=0.4.2,<0.5.0',
 'typer[all]>=0.6.1,<0.7.0']

entry_points = \
{'console_scripts': ['blackbricks = blackbricks.cli:app']}

setup_kwargs = {
    'name': 'blackbricks',
    'version': '1.0.0',
    'description': 'Black for Databricks notebooks',
    'long_description': "[![PyPI version](https://img.shields.io/pypi/v/blackbricks.svg?logo=pypi&logoColor=FFE873)](https://pypi.org/project/blackbricks/)\n[![Downloads](https://pepy.tech/badge/blackbricks)](https://pepy.tech/project/blackbricks)\n[![Downloads per month](https://pepy.tech/badge/blackbricks/month)](https://pepy.tech/project/blackbricks/month)\n[![License](https://img.shields.io/pypi/l/blackbricks)](LICENSE)\n[![Code style: Black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/psf/black)\n\n# Blackbricks\n\nA formatting tool for your Databricks notebooks.\n\n- Python cells are formatted with [black](https://github.com/psf/black)\n- SQL cells are formatted with [sqlparse](https://github.com/andialbrecht/sqlparse)\n\n## Breaking changes with version 1.0\n\nEarlier versions of blackbricks applied a patched version of black in order to allow two-space indentation. This was\ndone because Databricks used two-space indentation, and did not allow you to change that. \n\nSince then, Databricks has added the option to choose. Because you can now choose, blackbricks re-joins black in being\nuncompromising, and since version 1.0 you can no longer choose anything but 4 space indentation.\n\nIf you _must_ keep using two-space indentation, then stick to versions `<1.0`.\n\n## Table of Contents\n\n* [Installation](#installation)\n* [Usage](#usage)\n* [Version control integration](#version-control-integration)\n* [Contributing](#contributing)\n* [FAQ](#faq)\n\n## Installation\n\nWhile you can use `pip` directly, you should prefer using [pipx](https://pypa.github.io/pipx/).\n\n```bash\n$ pipx install blackbricks\n```\n\nYou probably also want to have installed the `databricks-cli`, in order to use `blackbricks` directly on your notebooks.\n\n``` bash\n$ pipx install databricks-cli\n$ databricks configure  # Required in order to use `blackbricks` on remote notebooks.\n```\n\n## Usage\nYou can use `blackbricks` on Python notebook files stored locally, or directly on the notebooks stored in Databricks. \n\nFor the most part, `blackbricks` operates very similarly to `black`.\n\n``` bash\n$ blackbricks notebook1.py notebook2.py  # Formats both notebooks.\n$ blackbricks notebook_directory/  # Formats every notebook under the directory (recursively).\n```\nAn important difference is that `blackbricks` will ignore any file that does not contain the `# Databricks notebook\nsource` header on the first line. Databricks adds this line to all Python notebooks. This means you can happily run\n`blackbricks` on a directory with both notebooks and regular Python files, and `blackbricks` won't touch the latter.\n\nIf you specify the `-r` or `--remote` flag, `blackbricks` will work directly on your notebooks stored in Databricks.\n\n``` bash\n$ blackbricks --remote /Users/username/notebook.py\n$ blackbricks --remote /Repos/username/repo-name/notebook.py\n```\n\n### Full usage\n\n```text\n$ poetry run blackbricks --help\n\n Usage: blackbricks [OPTIONS] [FILENAMES]...\n\n Formatting tool for Databricks python notebooks.\n Python cells are formatted using `black`, and SQL cells are formatted by `sqlparse`.\n Local files (without the `--remote` option):\n - Only files that look like Databricks (Python) notebooks will be processed. That is,\n they must start with the header `# Databricks notebook source`\n - If you specify a directory as one of the file names, all files in that directory will\n be added, including any subdirectory.\n Remote files (with the `--remote` option):\n - Make sure you have installed the Databricks CLI (``pip install databricks_cli``)\n - Make sure you have configured at least one profile (`databricks configure`). Check the\n file `~/.databrickscfg` if you are not sure.\n - File paths should start with `/`. Otherwise they are interpreted as relative to\n `/Users/username`, where `username` is the username specified in the Databricks profile\n used.\n\n╭─ Arguments ────────────────────────────────────────────────────────────────────────────╮\n│   filenames      [FILENAMES]...  Path to the notebook(s) to format. [default: None]    │\n╰────────────────────────────────────────────────────────────────────────────────────────╯\n╭─ Options ──────────────────────────────────────────────────────────────────────────────╮\n│ --remote                     -r                             If this option is used,    │\n│                                                             all filenames are treated  │\n│                                                             as paths to notebooks on   │\n│                                                             your Databricks host (i.e. │\n│                                                             not local files).          │\n│ --profile                    -p                    NAME     If using --remote, which   │\n│                                                             Databricks profile to use. │\n│                                                             [default: DEFAULT]         │\n│ --line-length                                      INTEGER  How many characters per    │\n│                                                             line to allow.             │\n│                                                             [default: 88]              │\n│ --sql-upper                      --no-sql-upper             SQL keywords should be     │\n│                                                             UPPERCASE or lowercase.    │\n│                                                             [default: sql-upper]       │\n│ --check                                                     Don't write the files      │\n│                                                             back, just return the      │\n│                                                             status. Return code 0      │\n│                                                             means nothing would        │\n│                                                             change.                    │\n│ --diff                                                      Don't write the files      │\n│                                                             back, just output a diff   │\n│                                                             for each file on stdout.   │\n│ --version                                                   Display version            │\n│                                                             information and exit.      │\n│ --help                                                      Show this message and      │\n│                                                             exit.                      │\n╰────────────────────────────────────────────────────────────────────────────────────────╯\n```\n\n\n\n## Version control integration\n\nUse [pre-commit](https://pre-commit.com). Add a `.pre-commit-config.yaml` file\nto your repo with the following content (changing/removing the `args` as you\nwish): \n\n```yaml\nrepos:\n-   repo: https://github.com/inspera/blackbricks\n    rev: 1.0.0\n    hooks:\n    - id: blackbricks\n      args: [--line-length=120]\n```\n\nSet the `rev` attribute to the most recent version of `blackbricks`.\nThe `args` are optional and can be used to set any of `blackbricks` options.\n\n## Contributing\n\nIf you find blackbricks useful, feel free to say so with a star. If you think it is utterly broken, you are more than\nwelcome to contribute improvements. Please open an issue first to discuss what you want added/fixed. Unless you are just\nadding tests. In that case your pull request is extremely likely to be merged right away.\n\n## FAQ\n\n### Can I disable SQL formatting?\n\nSure! Certain SQL statements might not be parsed and indented properly by `sqlparse`, and the result can be jumbled\nformatting. You can disable SQL formatting for a cell by adding `-- nofmt` to the very first line of a cell:\n\n```sql\n%sql  -- nofmt\nselect this,\n             sql_will,   -- be kept just\n         like_this\n  from if_that_is.what_you_need\n```\n\n### How do I use `blackbricks` on my Databricks notebooks?\n\nFirst, make sure you have set up `databricks-cli` on your system (see [installation](#installation)), and that you have\nat least one profile setup in `~/.databrickscfg`. As an example:\n\n```cfg\n# File: ~/.databrickscfg\n\n[DEFAULT]\nhost = https://dbc-b23456-a1243.cloud.databricks.com/\nusername = username@example.com\npassword = dapi12345678901234567890\n\n[OTHERPROFILE]\nhost = https://dbc-c54321-d234.cloud.databricks.com\nusername = name.user@example.com\npassword = dapi09876543211234567890\n```\n\nYou should use [access tokens](https://docs.databricks.com/dev-tools/api/latest/authentication.html) instead of your actual password.\n\nYou can then do:\n\n``` bash\n$ blackbricks --remote /Users/username@example.com/notebook.py  # Uses DEFAULT profile.\n$ blackbricks --remote notebook.py  # Equivalent to the above.\n$ blackbricks --remote --profile OTHERPROFILE /Users/name.user@example.com/notebook.py\n$ blackbricks --remote --profile OTHERPROFILE notebook.py  # Equivalent to the above.\n$ blackbricks --remote /Repos/username@example.com/repo-name/notebook.py  # Targeting notebook in a Repo\n```\n\n### Can you run blackbricks while using databricks in the browser?\n\nSee https://github.com/inspera/blackbricks/issues/27.\n\n### I get an error: `TypeError: init() got an unexpected keyword argument 'no_args_is_help'`\n\nThis means you had an old version of `click` installed from before, and your installation didn't upgrade it\nautomatically. Updating your installation should do the trick, e.g. `pip install -U blackbricks` or similar depending on\nyour installation method of choice.\n\n\n### Shell commands like `!ls` throws an error\n\nSee #21.\n",
    'author': 'Bendik Samseth',
    'author_email': 'bendik.samseth@inspera.no',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/inspera/blackbricks',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
